@@ -37,15 +37,17 @@
 
 (defcustom org-node-context-persist-on-disk nil
   "Whether to sync cached backlink previews to disk.
+The disk location is `org-node-data-dir'.
 
 This allows the context buffer created by \\[org-node-context-raise] to
 show up more instantly, even the first time it renders a given set of
 backlinks.
 
-Noticeable mainly if you are a connoisseur of low input latency,
-have a bad computer, and often re-start Emacs.
-
-For the cache location, see `org-node-data-dir'."
+Here\\='s how you test whether this is noticeable on your system:
+1. Open a file with many subtree nodes
+2. Collapse them all (i.e. S-TAB until you see only the headings)
+3. Have a context buffer open with `org-node-context-follow-mode'
+4. Hold the C-n or <down> key."
   :type 'boolean
   :package-version '(org-node . "2.0.0"))
 
@@ -286,10 +288,10 @@ properties.  Org-mode is enabled, but the org-element cache is not."
   :type 'hook
   :package-version '(org-node . "2.0.0"))
 
-(defcustom org-node-context-main-buffer "*Backlinks*"
+(defcustom org-node-context-main-buffer "*org-node context*"
   "Name of the main context buffer."
   :type 'string
-  :package-version '(org-node . "2.0.0"))
+  :package-version '(org-node . "3.13.0"))
 
 ;;;###autoload
 (define-minor-mode org-node-context-follow-local-mode
@@ -322,7 +324,7 @@ properties.  Org-mode is enabled, but the org-element cache is not."
                    ;; node as the "value" at that section.
                    value-atpt
                  (setq link-pos (org-mem-link-pos value-atpt))
-                 (org-mem-entry-by-id (org-mem-link-nearby-id value-atpt)))))
+                 (org-node-context--get-link-origin value-atpt))))
     (org-node-goto node)
     (when link-pos
       (goto-char link-pos)
@@ -454,14 +456,12 @@ that buffer."
         (setq header-line-format
               (concat "Context for \"" (org-mem-entry-title node) "\""))
         (magit-insert-section (org-node-context node)
-          (when-let* ((links (seq-filter #'org-mem-link-nearby-id
-                                         (org-mem-id-links-to-entry node))))
+          (when-let* ((links (org-mem-id-links-to-entry node)))
             (magit-insert-section (org-node-context 'id-links)
               (magit-insert-heading "ID backlinks:")
               (org-node-context--insert-backlink-sections links)
               (insert "\n")))
-          (when-let* ((links (seq-filter #'org-mem-link-nearby-id
-                                         (org-mem-roam-reflinks-to-entry node))))
+          (when-let* ((links (org-mem-roam-reflinks-to-entry node)))
             (magit-insert-section (org-node-context 'reflinks)
               (magit-insert-heading "Ref backlinks:")
               (org-node-context--insert-backlink-sections links)
@@ -472,28 +472,30 @@ that buffer."
 (defun org-node-context--insert-backlink-sections (links)
   "Insert a section displaying a preview of LINK."
   (dolist (link (sort links #'org-node-context--origin-title-lessp))
-    (let* ((node (org-mem-entry-by-id (org-mem-link-nearby-id link)))
-           (breadcrumbs (if-let* ((olp (org-mem-olpath-with-file-title node)))
+    (let* ((entry (org-node-context--get-link-origin link))
+           (breadcrumbs (if-let* ((olp (org-mem-olpath-with-file-title entry)))
                             (string-join olp " > ")
                           "Top")))
       (magit-insert-section (org-node-context link)
         (magit-insert-heading
           (format "%s (%s)"
-                  (propertize (org-mem-title node)
+                  (propertize (org-mem-title entry)
                               'face
                               'org-node-context-origin-title)
                   (propertize breadcrumbs 'face 'org-node-parent)))
-        (insert (org-node-context--get-preview node link))
+        (insert (org-node-context--get-preview link))
         (insert "\n")))))
 
 (defvar org-node-context--snippet-link)
-(defun org-node-context--get-preview (node link)
-  "Get a preview snippet out of NODE file, where LINK is.
+(defun org-node-context--get-preview (link)
+  "Get a preview snippet from around LINK in its containing file.
 
 Actually, if a snippet was previously cached, return the cached version,
-else briefly visit the file at LINK-POS and call
+else briefly visit the file, go to the link and call
 `org-node-context--extract-entry-at-point'."
-  (let* ((id (org-mem-entry-id node))
+  (let* ((entry (org-node-context--get-link-origin link))
+         ;; TODO: Change to upcoming `org-mem-entry-pseudo-id'
+         (eid (or (org-mem-entry-id entry) entry))
          (link-pos (org-mem-link-pos link))
          ;; NOTE: `pos-diff' is not necessary in a simple implementation, but
          ;; this level of granularity lets us avoid wiping all cached previews
@@ -504,12 +506,12 @@ else briefly visit the file at LINK-POS and call
          ;; Instead, we just don't wipe anything, and trust in a sloppy rule of
          ;; thumb: when the text between a link and its heading get edited,
          ;; that will almost always result in a new unique `pos-diff'.
-         (pos-diff (- link-pos (org-mem-entry-pos node))))
-    (or (alist-get pos-diff (gethash id org-node-context--previews))
+         (pos-diff (- link-pos (org-mem-entry-pos entry))))
+    (or (alist-get pos-diff (gethash eid org-node-context--previews))
         (setf
-         (alist-get pos-diff (gethash id org-node-context--previews))
+         (alist-get pos-diff (gethash eid org-node-context--previews))
          (let (snippet)
-           (with-current-buffer (org-node--work-buffer-for (org-mem-file node))
+           (with-current-buffer (org-node--work-buffer-for (org-mem-file entry))
              (goto-char link-pos)
              (setq snippet (org-node-context--extract-entry-at-point)))
            (with-current-buffer (org-mem-org-mode-scratch)
@@ -542,8 +544,18 @@ Decide this by getting the titles of the nodes wherein the links were
 found, and checking if the first title would come lexicographically
 before the second title."
   (string<
-   (org-mem-entry-title (org-mem-entry-by-id (org-mem-link-nearby-id link-1)))
-   (org-mem-entry-title (org-mem-entry-by-id (org-mem-link-nearby-id link-2)))))
+   (org-mem-entry-title (org-node-context--get-link-origin link-1))
+   (org-mem-entry-title (org-node-context--get-link-origin link-2))))
+
+(defun org-node-context--get-link-origin (link)
+  "Return the entry that contains LINK.
+Actually return the ancestor ID-node, if there is one and the direct
+entry has no ID."
+  (let ((id (org-mem-link-nearby-id link)))
+    (or (and id (org-mem-entry-by-id id))
+        ;; TODO: Change to upcoming `org-mem-link-entry'
+        (org-mem-entry-at-pos-in-file (org-mem-link-pos link)
+                                      (org-mem-link-file link)))))
 
 (provide 'org-node-context)
 
